#include "extension.h"

IEngineSound *enginesound = NULL;

CEngineSound g_EngineSound;

SMEXT_LINK(&g_EngineSound);

bool CEngineSound::SDK_OnLoad(char *error, size_t maxlength, bool late)
{
	g_pShareSys->AddNatives(myself, enginesound_natives);
	return true;
}


bool CEngineSound::SDK_OnMetamodLoad(ISmmAPI *ismm, char *error, size_t maxlen, bool late)
{
	GET_V_IFACE_ANY(engineFactory, enginesound, IEngineSound, IENGINESOUND_SERVER_INTERFACE_VERSION);
	return true;
}


static cell_t sm_PrefetchSound(IPluginContext *pCtx, const cell_t *pParams)
{
	if(enginesound && pParams[0] >= 1)
	{
		char *pSample = NULL;
		pCtx->LocalToString(pParams[1], &pSample);
		enginesound->PrefetchSound(pSample);
	}

	return 0;
}


static cell_t sm_GetSoundDuration(IPluginContext *pCtx, const cell_t *pParams)
{
	if(enginesound && pParams[0] >= 1)
	{
		char *pSample = NULL;
		pCtx->LocalToString(pParams[1], &pSample);
		return sp_ftoc(enginesound->GetSoundDuration(pSample));
	}

	return 0;
}

void EmitSound(IPluginContext *pCtx, const cell_t *pParams, int iType)
{
	if(enginesound && pParams[0] >= 13)
	{
		CFilter filter;

		if(pParams[1] == 0) filter.AddAll();
		else filter.Add(pParams[1]);

		int iEntity = pParams[2];
		int iChannel = pParams[3];

		char *pSample = NULL;
		if(iType != 3) pCtx->LocalToString(pParams[4], &pSample);

		int iSentence = 0;
		if(iType == 3) iSentence = pParams[4];

		float fVolume = sp_ctof(pParams[5]);

		float fAttenuation = 0.0f;
		int iLevel = 0;

		if(iType == 1) fAttenuation = sp_ctof(pParams[6]);
		else iLevel = pParams[6];

		int iFlags = pParams[7];
		int iPitch = pParams[8];

		cell_t *pCell = NULL;
		pCtx->LocalToPhysAddr(pParams[9], &pCell);

		Vector vecOrigin;
		Vector *pOrigin = &vecOrigin;

		vecOrigin.x = sp_ctof(pCell[0]);
		vecOrigin.y = sp_ctof(pCell[1]);
		vecOrigin.z = sp_ctof(pCell[2]);

		if(vecOrigin.x == 0.0f && vecOrigin.y == 0.0f && vecOrigin.z == 0.0f)
			pOrigin = NULL;

		pCell = NULL;
		pCtx->LocalToPhysAddr(pParams[10], &pCell);

		Vector vecDirection;
		Vector *pDirection = &vecDirection;

		vecDirection.x = sp_ctof(pCell[0]);
		vecDirection.y = sp_ctof(pCell[1]);
		vecDirection.z = sp_ctof(pCell[2]);

		if(vecDirection.x == 0.0f && vecDirection.y == 0.0f && vecDirection.z == 0.0f)
			pDirection = NULL;

		bool bUpdate = pParams[11];
		float fTime = sp_ctof(pParams[12]);
		int iSpeaker = pParams[13];

		if(iType == 1) enginesound->EmitSound(filter, iEntity, iChannel, pSample, fVolume, fAttenuation, iFlags, iPitch, pOrigin, pDirection, NULL, bUpdate, fTime, iSpeaker);
		else if(iType == 2) enginesound->EmitSound(filter, iEntity, iChannel, pSample, fVolume, (soundlevel_t)iLevel, iFlags, iPitch, pOrigin, pDirection, NULL, bUpdate, fTime, iSpeaker);
		else if(iType == 3) enginesound->EmitSentenceByIndex(filter, iEntity, iChannel, iSentence, fVolume, (soundlevel_t)iLevel, iFlags, iPitch, pOrigin, pDirection, NULL, bUpdate, fTime, iSpeaker);
	}
}


static cell_t sm_EmitSound(IPluginContext *pCtx, const cell_t *pParams)
{
	EmitSound(pCtx, pParams, 1);
	return 0;
}


static cell_t sm_EmitSoundL(IPluginContext *pCtx, const cell_t *pParams)
{
	EmitSound(pCtx, pParams, 2);
	return 0;
}


static cell_t sm_EmitSentenceByIndex(IPluginContext *pCtx, const cell_t *pParams)
{
	EmitSound(pCtx, pParams, 3);
	return 0;
}


static cell_t sm_StopSound(IPluginContext *pCtx, const cell_t *pParams)
{
	if(enginesound && pParams[0] >= 3)
	{
		int iEntity = pParams[1];
		int iChannel = pParams[2];

		char *pSample = NULL;
		pCtx->LocalToString(pParams[3], &pSample);

		enginesound->StopSound(iEntity, iChannel, pSample);
	}

	return 0;
}


static cell_t sm_EmitAmbientSound(IPluginContext *pCtx, const cell_t *pParams)
{
	if(enginesound && pParams[0] >= 4)
	{
		char *pSample = NULL;
		pCtx->LocalToString(pParams[1], &pSample);

		float fVolume = sp_ctof(pParams[2]);
		int iPitch = pParams[3];
		int iFlags = pParams[4];
		float fTime = sp_ctof(pParams[5]);

		enginesound->EmitAmbientSound(pSample, fVolume, iPitch, iFlags, fTime);
	}

	return 0;
}


const sp_nativeinfo_t enginesound_natives[] = 
{
	{"PrefetchSound",			sm_PrefetchSound},
	{"GetSoundDuration",		sm_GetSoundDuration},
	{"EmitSound",				sm_EmitSound},
	{"EmitSoundL",				sm_EmitSoundL},
	{"EmitSentenceByIndex",		sm_EmitSentenceByIndex},
	{"StopSound",				sm_StopSound},
	{"EmitAmbientSound",		sm_EmitAmbientSound},
	{NULL,						NULL},
};
